@@ -10,10 +10,11 @@ import sys
 import os
 import re
 import json
+import subprocess
 import pandas as pd
 
 from goatools.base import download_go_basic_obo
-from goatools.base import download_ncbi_associations
+#from goatools.base import download_ncbi_associations
 from goatools.obo_parser import GODag, OBOReader
 from goatools.gosubdag.gosubdag import GoSubDag
 from goatools.gosubdag.plot.gosubdag_plot import GoSubDagPlot
@@ -406,6 +407,28 @@ gi.email("youremail@address.com)
 """, file=sys.stderr)
         return
 
+def download_ncbi_associations(prt=sys.stdout):
+
+    os.environ["ftp_proxy"] = "http://proxyserv:3128"
+
+    process = subprocess.Popen(['wget', '-nv', '-O', 'gene2go.gz', 'ftp://ftp.ncbi.nlm.nih.gov/gene/DATA/gene2go.gz'],
+                        stdout=subprocess.PIPE, 
+                        stderr=subprocess.PIPE)
+    stdout, stderr = process.communicate()
+    print(stdout.decode(), file=prt)
+    print(stderr.decode(), file=prt)
+    assert not process.returncode
+
+    process = subprocess.Popen(['gzip', '-f', '-d', 'gene2go.gz'],
+                        stdout=subprocess.PIPE, 
+                        stderr=subprocess.PIPE)
+    stdout, stderr = process.communicate()
+    print(stdout.decode(), file=prt)
+    print(stderr.decode(), file=prt)    
+    assert not process.returncode
+    return 'gene2go'
+
+
 def fetch_background_genes(taxid=9606):
     
     _assert_entrez_email()
@@ -446,9 +469,7 @@ def fetch_background_genes(taxid=9606):
                         pass
   
     
-def get_genes_for_go_regex(regex, taxid=9606):
-
-    _assert_entrez_email()
+def get_terms_for_go_regex(regex, taxid=9606, add_children=False):
 
     try:
         taxid = int(taxid)
@@ -463,9 +484,7 @@ def get_genes_for_go_regex(regex, taxid=9606):
         gene2go = download_ncbi_associations(prt=null)
 
         objanno = Gene2GoReader("gene2go", taxids=[taxid], prt=null)
-        # go2geneids_human = objanno.get_id2gos(namespace='BP', go2geneids=True)
         go2geneids = objanno.get_id2gos(namespace='*', go2geneids=True, prt=null)
-
         srchhelp = GoSearch("go-basic.obo", go2items=go2geneids, log=null)
 
         results_all = re.compile(r'({})'.format(regex), flags=re.IGNORECASE)
@@ -474,7 +493,28 @@ def get_genes_for_go_regex(regex, taxid=9606):
         gos_all = srchhelp.get_matching_gos(results_all, prt=null)
         gos_no = srchhelp.get_matching_gos(results_not, gos=gos_all)
         gos = gos_all.difference(gos_no)
-        gos_all_with_children = srchhelp.add_children_gos(gos)
+        if add_children:
+            gos = srchhelp.add_children_gos(gos)
+
+        return list(gos)
+
+def get_genes_for_go_regex(regex, taxid=9606):
+
+    _assert_entrez_email()
+
+    try:
+        taxid = int(taxid)
+    except ValueError:
+        handle = Entrez.esearch(db="taxonomy", term=f'"{taxid}"[Scientific Name]')
+        taxid = int(Entrez.read(handle)['IdList'][0])
+        
+    with open(os.devnull, 'w') as null, redirect_stdout(null):
+
+        gos_all_with_children = get_terms_for_go_regex(regex, taxid=taxid, add_children=True)
+
+        objanno = Gene2GoReader("gene2go", taxids=[taxid], prt=null)
+        go2geneids = objanno.get_id2gos(namespace='*', go2geneids=True, prt=null)
+        srchhelp = GoSearch("go-basic.obo", go2items=go2geneids, log=null)
         geneids = srchhelp.get_items(gos_all_with_children)
 
         ncbi_tsv = f'{taxid}_protein_genes.txt'
@@ -737,7 +777,7 @@ def get_go_terms_for_genes(genes, taxid=9606, evidence=None):
     return list(sorted(df.GO_ID.unique().tolist()))
 
     
-def show_go_dag_for_terms(terms):
+def show_go_dag_for_terms(terms, add_relationships=True):
     
     with open(os.devnull, 'w') as null, redirect_stdout(null):
 
@@ -749,7 +789,11 @@ def show_go_dag_for_terms(terms):
         file_gene2go = download_ncbi_associations(prt=null)
 
         #Load Ontologies
-        obodag = GODag("go-basic.obo", optional_attrs=['relationship', 'def'], prt=null)
+        if add_relationships:
+            optional_attrs=['relationship', 'def']
+        else:
+            optional_attrs=['def']
+        obodag = GODag("go-basic.obo", optional_attrs=optional_attrs, prt=null)
 
         gosubdag = GoSubDag(terms, obodag, relationships=True)
         GoSubDagPlot(gosubdag).plt_dag('plot.png')
@@ -812,37 +856,57 @@ Inferred from Electronic Annotation (IEA)
     display(Markdown(s))  
                     
         
+def  _write_go_hdf():
+
+    with open(os.devnull, 'w') as null, redirect_stdout(null):
+
+        if not os.path.exists('go-basic.h5'):
+
+            # Get http://geneontology.org/ontology/go-basic.obo
+            obo_fname = download_go_basic_obo(prt=null)
+
+            # Download Associations, if necessary
+            # Get ftp://ftp.ncbi.nlm.nih.gov/gene/DATA/gene2go.gz
+            file_gene2go = download_ncbi_associations(prt=null)
+
+            r = OBOReader(optional_attrs=['def'])
+            rows = [(e.id, e.name, e.defn) for e in OBOReader(optional_attrs=['def'])]
+            df = pd.DataFrame().from_records(rows, columns=['goterm', 'goname', 'description'])
+            df.to_hdf('go-basic.h5', key='df', format='table', data_columns=['goterm', 'goname'])
+
+
+def go_term2name(term):
+
+    _write_go_hdf()
+    with pd.HDFStore('go-basic.h5', 'r') as store:
+        entry = store.select("df", "goterm == %r" % term).iloc[0]
+    return entry.goterm
+
+
+def go_name2term(name):
+
+    _write_go_hdf()
+    with pd.HDFStore('go-basic.h5', 'r') as store:
+        entry = store.select("df", "goname == %r" % name.lower()).iloc[0]
+    return entry.goterm
+
+
 def go_info(terms):
             
     if type(terms) is not list:
         terms = [terms]
 
-    with open(os.devnull, 'w') as null, redirect_stdout(null):
+    _write_go_hdf()
 
-        # Get http://geneontology.org/ontology/go-basic.obo
-        obo_fname = download_go_basic_obo(prt=null)
-
-        # Download Associations, if necessary
-        # Get ftp://ftp.ncbi.nlm.nih.gov/gene/DATA/gene2go.gz
-        file_gene2go = download_ncbi_associations(prt=null)
-
-        if not os.path.exists('go-basic.h5'):
-
-            r = OBOReader(optional_attrs=['def'])
-            rows = [(e.id, e.name, e.defn) for e in OBOReader(optional_attrs=['def'])]
-            df = pd.DataFrame().from_records(rows, columns=['goterm', 'goname', 'description']).set_index('goterm')
-            df.to_hdf('go-basic.h5', key='df', format='table')
-
-    store = pd.HDFStore('go-basic.h5')
-
-    for term in terms:
-        entry = store.select("df", "index == %r" % term).iloc[0]
-        desc = re.search(r'"([^"]+)"', entry.description).group(1)
-        s = f'**<span style="color:gray;">{term}:</span>** **{entry.goname}**  \n {desc}    \n\n ----'        
-        display(Markdown(s))
+    with pd.HDFStore('go-basic.h5', 'r') as store:
+        for term in terms:
+            entry = store.select("df", "goterm == %r" % term.upper()).iloc[0]
+            desc = re.search(r'"([^"]+)"', entry.description).group(1)
+            s = f'**<span style="color:gray;">{term}:</span>** **{entry.goname}**  \n {desc}    \n\n ----'        
+            display(Markdown(s))
 
 
-def go_enrichment(gene_list, taxid=9606, background_chrom=None):
+def go_enrichment(gene_list, taxid=9606, background_chrom=None, terms=None):
 
     _assert_entrez_email()
 
@@ -866,6 +930,14 @@ def go_enrichment(gene_list, taxid=9606, background_chrom=None):
         #Load Ontologies
         obodag = GODag("go-basic.obo", optional_attrs=['relationship', 'def'], prt=null)
 
+        # limit go dag to a sub graph including only specified terms and their children
+        if terms is not None:
+
+            srchhelp = GoSearch("go-basic.obo", go2items=go2geneids, log=null)
+            terms_with_children = srchhelp.add_children_gos(terms)
+
+            obodag = GoSubDag(terms_with_children, obodag)
+
         # load associations
         # Read NCBI's gene2go. Store annotations in a list of namedtuples
         objanno = Gene2GoReader(file_gene2go, taxids=[taxid])
